
/**
 *
 *  Plugin loader class.
 *
 *  This is a singleton object.
 *
 * $Revision: 1.3 $
 * @author Steve Kemp
 */


#include <stdio.h>
#include <dirent.h>
#include <dlfcn.h>
#include "plugin.h"


CPluginLoader *CPluginLoader::m_pluginLoader = NULL;


CPluginLoader::CPluginLoader(std::string pluginDirectory, int debug ) 
{
  m_debug = debug;

  initAllPlugins( pluginDirectory );
}

CPluginLoader::~CPluginLoader()
{
  unloadAllPlugins();
}

CPluginLoader *CPluginLoader::getInstance( std::string pluginDirectory, int debug  )
{
  if ( m_pluginLoader == 0 )
  {
    m_pluginLoader = new CPluginLoader( pluginDirectory, debug );
  }
  return( m_pluginLoader );
}


int CPluginLoader::callPlugin( const char *src, const char *dst, connection *threadObject )
{
  std::vector< CLoadedPlugin * >::iterator i;

  for( i = m_plugins.begin(); i != m_plugins.end(); i++ )
    {
      CLoadedPlugin *found = *i;

      IPlugin *plug = (*found->factory)();
      if ( plug )
      {
	if ( m_debug )
	{
	  printf(" [Calling  '%s' -> by %s]\n",
		 found->description.c_str(), found->author.c_str() );
	}

	plug->processPacket( src, dst, threadObject );
      }
      delete( plug );
    }
  return 0;
}



/**
 * Load all our plugins.
 */
int CPluginLoader::initAllPlugins( std::string pluginDir )
{
    struct dirent **namelist = NULL;
    int i          = 0;
    int entries    = 0;

    /*
     * Read the elements in this directory.
     */
    entries = scandir( pluginDir.c_str(), &namelist, NULL, alphasort);
    if ( entries == -1 )
    {
        if ( m_debug )
	  {
	    printf( " [Cannot read directory '%s']\n", 
		    pluginDir.c_str() ); }

        return 0;
    }

    for ( i = 0; i < entries; i++ )
    {
        struct dirent *dirent = NULL;
        dirent  = namelist[ i ];

        if ( dirent->d_name[ 0 ] != '.' )
	  {
	    std::string path = pluginDir;
	    path += "/";
	    path += dirent->d_name;

	    /*
	     * init *.so in the modules directory.
	     */
	    if ( strcmp( path.substr( path.size() - 3 ).c_str(), ".so" ) == 0 )
	    {
	      initPlugin( path.c_str() );
	    }
	  }
    }

    return 1;
}


/**
 * Unload each plugin.
 */
void CPluginLoader::unloadAllPlugins()
{
  std::vector< CLoadedPlugin * >::iterator i;

  for( i = m_plugins.begin(); i != m_plugins.end(); i++ )
    {
      CLoadedPlugin *f = *i;

      if ( m_debug )
      {
	printf(" [Unloading -> %s]\n",
	       f->pluginFilename.c_str() ); 
      }

      dlclose( f->handle );
    }
}

/**
 * Return all the plugins which are loaded.
 */
std::vector< CLoadedPlugin * > CPluginLoader::getAllPlugins()
{
  return( m_plugins );
}


/**
 * Initialize a given plugin.
 */
void CPluginLoader::initPlugin( const char *name )
{
  CLoadedPlugin *first = new CLoadedPlugin();

  /* Load in the shared library
   */
  first->handle = dlopen(name, RTLD_LAZY);
  if ( first->handle == NULL) 
  {
    if ( m_debug )
    { 
      fprintf(stderr, " [Unable to open (%s): %s]\n", 
	      name, dlerror()); 
    }

    delete( first );
    return;
  }


  /* get the address of the factory function we'll want for later.
   */
  first->factory = (IPlugin * (*)()) dlsym( first->handle,
					    "factory");
  if (!first->factory)
  {
    delete( first );

    if ( m_debug )
    { 
      fprintf(stderr, " [Plugin library (%s) : No factory method!]\n",
	      name );
    }
    return;
  }    

  /*
   * Create instance of the plugin.
   */
  IPlugin *i = (*first->factory)();
  if ( i )
  {
    /*
     * Save the details of this plugin.
     */
    first->author         = i->getAuthor();
    first->description    = i->getDescription();
    first->pluginFilename = name;

    m_plugins.push_back( first );

    if ( m_debug )
    { 
      printf(" [Loaded -> %s]\n", 
	     name ); 
    }

    delete( i );
  }
}
