
#ifndef _CPLUGIN_H
#define _CPLUGIN_H 1

#include <string>
#include <vector>

#include "iplugin.h"

/*
 * Objects of this class hold all the plugins we know about.
 *
 */
class CLoadedPlugin
{
public:
  /* Handle to the open library. */
  void *handle;

  /* Author of the plugin */
  std::string author;
  
  /* Description of the plugin. */
  std::string description;

  /* The name of the library which this object contains. */
  std::string pluginFilename;

  /*
   *  Pointer to the factory method which can be used to create an
   * instance of this plugin object.
   */
  IPlugin *(*factory)();

};


/*
 * This class encapsulates the loading of plugin modules.
 *
 * It takes a directory to scan, and then allows us to call
 * an unknown plugin which will handle a given path.
 *
 */
class CPluginLoader
{
 public:
  /**
   * Accessor to the CPluginLoader singleton.
   */
  static CPluginLoader *getInstance(std::string pluginDirectory = "",
		  		    int debug = 0 );

  /**
   * Destructor.
   */
  virtual ~CPluginLoader();

  /**
   * Call a plugin to handle the given path.
   * Note that we don't need to know which plugin handles this path,
   * that's all handled magically by the IPlugin interface.
   */
  int callPlugin( const char *src, const char *dst, connection *threadObject );

  /**
   * Return all the plugins we have loaded at the current time.
   */
  std::vector< CLoadedPlugin * > getAllPlugins();

 protected:

  /**
   * Protected constructor.  This is a singleton object.
   */
  CPluginLoader( std::string directory, int debug );

  /**
   * The single static plugin loader object.
   */
  static CPluginLoader *m_pluginLoader;

 private:
  /**
   * Load and initialize all plugins beneath our plugin directory.
   */
  int initAllPlugins( std::string pluginDir );

  /**
   * Unload all our loaded plugins.
   */
  void unloadAllPlugins();

  /**
   * Initialize + load the given plugin.
   * @param name The name of the plugin library.
   */
  void initPlugin( const char *name );

  /*
   * The list of plugins we've got loaded.
   */
  std::vector < CLoadedPlugin *> m_plugins;
 
  /*
   * Is debugging enabled?
   */
  int m_debug;

};


#endif /* _CPLUGIN_H */
